import time
from xml.etree.ElementTree import SubElement


def get_pub_date(poa_article, crossref_config, default_pub_date):
    """
    For using in XML generation, use the article pub date
    or by default use the run time pub date
    """
    pub_date = None

    for date_type in crossref_config.get("pub_date_types"):
        pub_date_obj = poa_article.get_date(date_type)
        if pub_date_obj:
            break

    if pub_date_obj:
        pub_date = pub_date_obj.date
    else:
        # Default use the run time date
        pub_date = default_pub_date
    return pub_date


def set_acceptance_date(parent, article_date):
    # article_date is an ArticleDate object
    if article_date:
        date_tag = SubElement(parent, "acceptance_date")
        set_date_detail(date_tag, article_date.date)


def set_publication_date(parent, pub_date):
    # pub_date is a python time object
    if pub_date:
        publication_date_tag = SubElement(parent, "publication_date")
        publication_date_tag.set("media_type", "online")
        set_date_detail(publication_date_tag, pub_date)


def set_date_detail(parent, pub_date):
    month_tag = SubElement(parent, "month")
    month_tag.text = str(pub_date.tm_mon).zfill(2)
    day_tag = SubElement(parent, "day")
    day_tag.text = str(pub_date.tm_mday).zfill(2)
    year_tag = SubElement(parent, "year")
    year_tag.text = str(pub_date.tm_year)


def iso_date_string(pub_date):
    """format a time.struct_time object into an iso format date string e.g. 2019-12-31"""
    return time.strftime("%Y-%m-%d", pub_date) if pub_date else None



from xml.etree.ElementTree import SubElement
from elifecrossref import (
    abstract,
    access_indicators,
    citation,
    component,
    contributor,
    crossmark,
    dataset,
    dates,
    doi,
    funding,
    preprint,
    related,
    title,
)


def set_journal_article(parent, poa_article, pub_date, crossref_config):
    journal_article_tag = SubElement(parent, "journal_article")
    journal_article_tag.set("publication_type", "full_text")
    if (
        crossref_config.get("reference_distribution_opts")
        and crossref_config.get("reference_distribution_opts") != ""
    ):
        journal_article_tag.set(
            "reference_distribution_opts",
            crossref_config.get("reference_distribution_opts"),
        )

    # Set the title with italic tag support
    title.set_titles(journal_article_tag, poa_article.title, crossref_config)

    contributor.set_article_contributors(
        journal_article_tag, poa_article, crossref_config.get("contrib_types")
    )

    abstract.set_abstract(journal_article_tag, poa_article, crossref_config)
    abstract.set_digest(journal_article_tag, poa_article, crossref_config)

    # Journal publication date
    dates.set_publication_date(journal_article_tag, pub_date)

    # article accepted date
    dates.set_acceptance_date(journal_article_tag, poa_article.get_date("accepted"))

    publisher_item_tag = SubElement(journal_article_tag, "publisher_item")
    if crossref_config.get("elocation_id") and poa_article.elocation_id:
        item_number_tag = SubElement(publisher_item_tag, "item_number")
        item_number_tag.set("item_number_type", "article_number")
        item_number_tag.text = poa_article.elocation_id
    identifier_tag = SubElement(publisher_item_tag, "identifier")
    identifier_tag.set("id_type", "doi")
    identifier_tag.text = poa_article.doi

    # Crossmark data includes funding and access indicators, otherwise add them separately
    if crossmark.do_crossmark(poa_article, crossref_config):
        crossmark.set_crossmark(journal_article_tag, poa_article, crossref_config)
    else:
        funding.set_fundref(journal_article_tag, poa_article)
        access_indicators.set_access_indicators(
            journal_article_tag, poa_article, crossref_config
        )

    # this is the spot to add the relations program tag if it is required
    relations_program_tag = None
    if related.do_relations_program(poa_article) is True:
        relations_program_tag = related.set_relations_program(
            journal_article_tag, relations_program_tag
        )

    dataset.set_datasets(relations_program_tag, poa_article)

    set_archive_locations(journal_article_tag, crossref_config.get("archive_locations"))

    doi.set_article_doi_data(journal_article_tag, poa_article, crossref_config)

    citation.set_citation_list(
        journal_article_tag, poa_article, relations_program_tag, crossref_config
    )

    component.set_component_list(journal_article_tag, poa_article, crossref_config)

    if related.do_preprint_related_item(poa_article) is True:
        preprint.set_preprint(relations_program_tag, poa_article.preprint)


def set_archive_locations(parent, archive_locations):
    if archive_locations:
        archive_locations_tag = SubElement(parent, "archive_locations")
        for archive_location in archive_locations:
            archive_tag = SubElement(archive_locations_tag, "archive")
            archive_tag.set("name", archive_location)



from xml.etree.ElementTree import SubElement
from elifearticle import utils as eautils
from elifecrossref import dates, journal_article


def set_journal(parent, poa_article, crossref_config, default_pub_date):
    # Add journal for each article
    journal_tag = SubElement(parent, "journal")
    set_journal_metadata(journal_tag, poa_article)

    journal_issue_tag = SubElement(journal_tag, "journal_issue")

    pub_date = dates.get_pub_date(poa_article, crossref_config, default_pub_date)
    dates.set_publication_date(journal_issue_tag, pub_date)

    journal_volume_tag = SubElement(journal_issue_tag, "journal_volume")
    volume_tag = SubElement(journal_volume_tag, "volume")
    # Use volume from the article unless not present then use the default
    if poa_article.volume:
        volume_tag.text = poa_article.volume
    else:
        if crossref_config.get("year_of_first_volume"):
            volume_tag.text = eautils.calculate_journal_volume(
                pub_date, crossref_config.get("year_of_first_volume")
            )

    # Add journal article
    journal_article.set_journal_article(
        journal_tag, poa_article, pub_date, crossref_config
    )


def set_journal_metadata(parent, poa_article):
    # journal_metadata
    journal_metadata_tag = SubElement(parent, "journal_metadata")
    journal_metadata_tag.set("language", "en")
    full_title_tag = SubElement(journal_metadata_tag, "full_title")
    full_title_tag.text = poa_article.journal_title
    set_issn_tag(journal_metadata_tag, poa_article)


def set_issn_tag(parent, poa_article):
    issn_tag = SubElement(parent, "issn")
    issn_tag.set("media_type", "electronic")
    issn_tag.text = poa_article.journal_issn



from xml.etree.ElementTree import SubElement
from elifecrossref import related, tags


def set_citation_list(parent, poa_article, relations_program_tag, crossref_config):
    """
    Set the citation_list from the article object ref_list objects
    """
    ref_index = 0
    if poa_article.ref_list:
        citation_list_tag = SubElement(parent, "citation_list")
    for ref in poa_article.ref_list:
        # Increment
        ref_index = ref_index + 1
        # decide whether to create a related_item for the citation
        if related.do_citation_related_item(ref):
            set_citation_related_item(relations_program_tag, ref)

        # continue with creating a citation tag
        set_citation(
            citation_list_tag,
            ref,
            ref_index,
            crossref_config.get("face_markup"),
            crossref_config.get("crossref_schema_version"),
        )


def set_citation(parent, ref, ref_index, face_markup, crossref_schema_version):
    # continue with creating a citation tag
    citation_tag = SubElement(parent, "citation")
    set_citation_key(citation_tag, ref, ref_index)
    set_citation_title(citation_tag, ref)
    set_citation_authors(citation_tag, ref)
    set_citation_volume(citation_tag, ref)
    set_citation_issue(citation_tag, ref)
    set_citation_first_page(citation_tag, ref)
    set_citation_year(citation_tag, ref)
    set_citation_article_title(citation_tag, ref)
    set_citation_doi(citation_tag, ref)
    set_citation_isbn(citation_tag, ref)
    set_elocation_id(citation_tag, ref, crossref_schema_version)
    # unstructured-citation
    if do_unstructured_citation(ref) is True:
        set_unstructured_citation(citation_tag, ref, face_markup)


def set_citation_key(citation_tag, ref, ref_index):
    if ref.id:
        citation_tag.set("key", ref.id)
    else:
        citation_tag.set("key", str(ref_index))


def set_citation_title(parent, ref):
    if ref.source:
        if ref.publication_type == "journal":
            journal_title_tag = SubElement(parent, "journal_title")
            journal_title_tag.text = ref.source
        else:
            volume_title_tag = SubElement(parent, "volume_title")
            volume_title_tag.text = ref.source


def set_citation_authors(parent, ref):
    authors = filter_citation_authors(ref)
    if authors:
        # Only set the first author surname
        first_author = authors[0]
        if first_author.get("surname"):
            author_tag = SubElement(parent, "author")
            author_tag.text = first_author.get("surname")
        elif first_author.get("collab"):
            tags.add_clean_tag(parent, "author", first_author.get("collab"))


def set_citation_volume(parent, ref):
    if ref.volume:
        volume_tag = SubElement(parent, "volume")
        volume_tag.text = ref.volume[0:31]


def set_citation_issue(parent, ref):
    if ref.issue:
        issue_tag = SubElement(parent, "issue")
        issue_tag.text = ref.issue


def set_citation_first_page(parent, ref):
    if ref.fpage:
        first_page_tag = SubElement(parent, "first_page")
        first_page_tag.text = ref.fpage


def set_citation_year(parent, ref):
    if ref.year or ref.year_numeric:
        cyear_tag = SubElement(parent, "cYear")
        # Prefer the numeric year value if available
        if ref.year_numeric:
            cyear_tag.text = str(ref.year_numeric)
        else:
            cyear_tag.text = ref.year


def set_citation_article_title(parent, ref):
    if ref.article_title or ref.data_title:
        if ref.article_title:
            tags.add_clean_tag(parent, "article_title", ref.article_title)
        elif ref.data_title:
            tags.add_clean_tag(parent, "article_title", ref.data_title)


def set_citation_doi(parent, ref):
    if ref.doi:
        doi_tag = SubElement(parent, "doi")
        doi_tag.text = ref.doi


def set_citation_isbn(parent, ref):
    if ref.isbn:
        isbn_tag = SubElement(parent, "isbn")
        isbn_tag.text = ref.isbn


def set_elocation_id(parent, ref, crossref_schema_version):
    if ref.elocation_id:
        if crossref_schema_version in ["4.3.5", "4.3.7", "4.4.0"]:
            # Until alternate tag is available, elocation-id goes into first_page tag
            first_page_tag = SubElement(parent, "first_page")
            first_page_tag.text = ref.elocation_id
        else:
            # schema greater than 4.4.0 supports elocation_id
            elocation_id_tag = SubElement(parent, "elocation_id")
            elocation_id_tag.text = ref.elocation_id


def set_citation_related_item(parent, ref):
    related_item_tag = SubElement(parent, "rel:related_item")
    if ref.data_title:
        related.set_related_item_description(related_item_tag, ref.data_title)
    identifier_type = None
    related_item_text = None
    related_item_type = "inter_work_relation"
    relationship_type = "references"
    if ref.doi:
        identifier_type = "doi"
        related_item_text = ref.doi
    elif ref.accession:
        identifier_type = "accession"
        related_item_text = ref.accession
    elif ref.pmid:
        identifier_type = "pmid"
        related_item_text = ref.pmid
    elif ref.uri:
        identifier_type = "uri"
        related_item_text = ref.uri
    if identifier_type and related_item_text:
        related.set_related_item_work_relation(
            related_item_tag,
            related_item_type,
            relationship_type,
            identifier_type,
            related_item_text,
        )


def set_unstructured_citation(parent, ref, face_markup):
    # tag_content
    tag_content = ""
    author_line = citation_author_line(ref)

    if ref.publication_type and ref.publication_type in [
        "confproc",
        "patent",
        "preprint",
        "report",
        "software",
        "thesis",
        "web",
        "webpage",
    ]:
        tag_content = ". ".join(
            [
                item.rstrip(".")
                for item in [
                    author_line,
                    ref.year,
                    ref.article_title,
                    ref.data_title,
                    citation_publisher(ref),
                    ref.source,
                    ref.version,
                    ref.patent,
                    ref.conf_name,
                    citation_uri(ref),
                ]
                if item is not None
            ]
        )
        tag_content += "."
    # add the tag if there is tag_content
    if tag_content != "":
        # handle inline tagging
        if face_markup is True:
            tags.add_inline_tag(parent, "unstructured_citation", tag_content)
        else:
            tags.add_clean_tag(parent, "unstructured_citation", tag_content)
    return parent


def citation_author_line(ref):
    author_line = None
    author_names = []
    # extract all authors regardless of their group-type
    for author in ref.authors:
        author_name = ""
        if author.get("surname"):
            author_name = author.get("surname")
            if author.get("given-names"):
                author_name += " " + author.get("given-names")
        elif author.get("collab"):
            author_name = author.get("collab")
        if author_name != "":
            author_names.append(author_name)
    if author_names:
        author_line = ", ".join(author_names)
    return author_line


def citation_publisher(ref):
    if ref.publisher_loc or ref.publisher_name:
        return ": ".join(
            [
                item
                for item in [ref.publisher_loc, ref.publisher_name]
                if item is not None
            ]
        )
    return None


def citation_uri(ref):
    uri_content = ""
    if ref.uri:
        uri_content = ref.uri
    if ref.date_in_citation:
        uri_content += " [Accessed " + ref.date_in_citation + "]"
    return uri_content if uri_content != "" else None


def do_unstructured_citation(ref):
    """decide if a citation should have an unstructured_citation tag added"""
    if ref.publication_type and ref.publication_type in [
        "confproc",
        "patent",
        "software",
        "thesis",
        "web",
        "webpage",
    ]:
        return True
    if (
        ref.publication_type
        and ref.publication_type in ["preprint"]
        and ref.doi is None
    ):
        return True
    if ref.publication_type and ref.publication_type in ["report"] and ref.isbn is None:
        return True
    return False


def filter_citation_authors(ref):
    """logic for which authors to select for citation records"""
    # First consider authors with group-type author
    authors = [c for c in ref.authors if c.get("group-type") == "author"]
    if not authors:
        # Take editors if there are no authors
        authors = [c for c in ref.authors if c.get("group-type") == "editor"]
    return authors



from elifetools import utils as etoolsutils
from elifetools import xmlio
from elifearticle import utils as eautils
from elifecrossref import utils


# namespaces for when reparsing XML strings
REPARSING_NAMESPACES = """ xmlns:jats="http://www.ncbi.nlm.nih.gov/JATS1"
 xmlns:mml="http://www.w3.org/1998/Math/MathML" xmlns:xlink="http://www.w3.org/1999/xlink" """


def clean_tags(original_string, do_not_clean=None):
    """remove all unwanted inline tags from the string"""
    do_not_clean_tags = do_not_clean if do_not_clean else []
    tag_converted_string = original_string
    for tag in utils.allowed_tags():
        if tag not in do_not_clean_tags:
            # first do exact tag replacements
            if tag.startswith("<") and tag.endswith(">"):
                tag_converted_string = tag_converted_string.replace(tag, "")
            # then replace by fragments for mml tags if present
            if tag.startswith("<") and not tag.endswith(">"):
                tag_fragment = tag.lstrip("</")
                tag_converted_string = eautils.remove_tag(
                    tag_fragment, tag_converted_string
                )
    remove_tags = ["inline-formula"]
    for tag in remove_tags:
        if tag not in do_not_clean_tags:
            tag_converted_string = eautils.remove_tag(tag, tag_converted_string)
    return tag_converted_string


def add_clean_tag(
    parent,
    tag_name,
    original_string,
    namespaces=REPARSING_NAMESPACES,
    attributes=None,
    attributes_text="",
):
    """remove allowed tags and then add a tag the parent"""
    tag_converted_string = clean_tags(original_string)
    tag_converted_string = etoolsutils.escape_ampersand(tag_converted_string)
    tag_converted_string = etoolsutils.escape_unmatched_angle_brackets(
        tag_converted_string
    )
    minidom_tag = xmlio.reparsed_tag(
        tag_name, tag_converted_string, namespaces, attributes_text
    )
    append_tag(parent, minidom_tag, attributes=attributes)


def add_inline_tag(
    parent,
    tag_name,
    original_string,
    namespaces=REPARSING_NAMESPACES,
    attributes=None,
    attributes_text="",
):
    """replace inline tags found in the original_string and then add a tag the parent"""
    tag_converted_string = convert_inline_tags(original_string)
    minidom_tag = xmlio.reparsed_tag(
        tag_name, tag_converted_string, namespaces, attributes_text
    )
    append_tag(parent, minidom_tag, attributes=attributes)


def append_tag(parent, minidom_tag, attributes=None):
    """given final minidom tag and details, append a tag to the parent tag"""
    xmlio.append_minidom_xml_to_elementtree_xml(
        parent, minidom_tag, attributes=attributes, child_attributes=True
    )


def convert_inline_tags(original_string):
    tag_converted_string = etoolsutils.escape_ampersand(original_string)
    tag_converted_string = etoolsutils.escape_unmatched_angle_brackets(
        tag_converted_string, utils.allowed_tags()
    )
    tag_converted_string = eautils.replace_tags(tag_converted_string, "italic", "i")
    tag_converted_string = eautils.replace_tags(tag_converted_string, "bold", "b")
    tag_converted_string = eautils.replace_tags(tag_converted_string, "underline", "u")
    return tag_converted_string



from xml.etree.ElementTree import Element, SubElement
from elifearticle import utils as eautils
from elifecrossref import related


def set_preprint(parent, preprint):
    """
    add rel:inter_work_relation tag for a preprint
    """
    related_item_tag = SubElement(parent, "rel:related_item")
    related_item_type = "intra_work_relation"
    relationship_type = "hasPreprint"
    if preprint.doi:
        identifier_type = "doi"
        related_item_text = preprint.doi
    elif preprint.uri:
        identifier_type = "uri"
        related_item_text = preprint.uri
    related.set_related_item_work_relation(
        related_item_tag,
        related_item_type,
        relationship_type,
        identifier_type,
        related_item_text,
    )



from xml.etree.ElementTree import SubElement
from elifecrossref import dates, journal, title


def set_pending_publication(parent, poa_article, crossref_config):

    pending_publication_tag = SubElement(parent, "pending_publication")
    publication_tag = SubElement(pending_publication_tag, "publication")
    publication_full_title_tag = SubElement(publication_tag, "full_title")
    publication_full_title_tag.text = crossref_config.get("registrant")

    journal.set_issn_tag(publication_tag, poa_article)

    title.set_titles(pending_publication_tag, poa_article.title, crossref_config)

    dates.set_acceptance_date(pending_publication_tag, poa_article.get_date("accepted"))

    doi_tag = SubElement(pending_publication_tag, "doi")
    doi_tag.text = poa_article.doi



from xml.etree.ElementTree import Element
from elifearticle import utils as eautils
from elifecrossref import tags


def set_titles(parent, title, crossref_config):
    """
    Set the titles and title tags allowing sub tags within title
    """
    root_tag_name = "titles"
    tag_name = "title"
    root_xml_element = Element(root_tag_name)
    # remove unwanted tags
    tag_converted_title = eautils.remove_tag("ext-link", title)
    if crossref_config.get("face_markup") is True:
        tags.add_inline_tag(root_xml_element, tag_name, tag_converted_title)
    else:
        tags.add_clean_tag(root_xml_element, tag_name, tag_converted_title)
    parent.append(root_xml_element)



from collections import OrderedDict
from xml.etree import ElementTree
from xml.etree.ElementTree import SubElement
import requests


# to convert to Crossref type values they accept
CONTENT_TYPE_MAP = {
    "pre-results": "preResults",
    "preResults": "preResults",
    "results": "results",
    "post-results": "postResults",
    "postResults": "postResults",
}


def do_clinical_trials(poa_article):
    return bool(hasattr(poa_article, "clinical_trials") and poa_article.clinical_trials)


def parse_registries_xml(registries_xml):
    """convert registries XML into a name to doi map"""
    name_to_doi_map = OrderedDict()
    tree = ElementTree.fromstring(registries_xml)
    # simplified namespace name for easier finding
    namespaces = {"xschema": "http://www.crossref.org/xschema/1.1"}
    for component in tree.findall(
        ".//xschema:component_list/xschema:component", namespaces
    ):
        name_tag = component.find(".//xschema:titles/xschema:subtitle", namespaces)
        doi_tag = component.find(".//xschema:doi_data/xschema:doi", namespaces)
        name_to_doi_map[name_tag.text] = doi_tag.text
    return name_to_doi_map


def registry_name_to_doi_map(registry_url):
    """get XML for clinical trials registries and turn into a name to doi map"""
    response = requests.get(registry_url)
    return parse_registries_xml(response.content)


def set_clinical_trials(parent, poa_article, crossref_config):
    if do_clinical_trials(poa_article):
        # get a map of name to registry DOI
        name_to_doi_map = (
            registry_name_to_doi_map(crossref_config.get("clinical_trials_registries"))
            if crossref_config.get("clinical_trials_registries")
            else None
        )
        ai_program_tag = set_ct_program(parent)
        for clinical_trial in poa_article.clinical_trials:
            clinical_trial_number = SubElement(
                ai_program_tag, "ct:clinical-trial-number"
            )
            clinical_trial_number.set(
                "registry", clinical_trial.get_registry_doi(name_to_doi_map)
            )
            if (
                clinical_trial.content_type
                and clinical_trial.content_type in CONTENT_TYPE_MAP
            ):
                clinical_trial_number.set(
                    "type", CONTENT_TYPE_MAP.get(clinical_trial.content_type)
                )
            clinical_trial_number.text = clinical_trial.document_id


def set_ct_program(parent):
    ct_program_tag = SubElement(parent, "ct:program")
    return ct_program_tag



from xml.etree.ElementTree import SubElement
from elifecrossref import access_indicators, resource_url


def set_collection(parent, poa_article, collection_property, crossref_config):
    if do_set_collection(poa_article, collection_property, crossref_config):
        if collection_property == "text-mining":
            collection_tag = SubElement(parent, "collection")
            collection_tag.set("property", collection_property)
            if do_set_collection_text_mining_pdf(poa_article, crossref_config) is True:
                item_tag = SubElement(collection_tag, "item")
                resource_tag = SubElement(item_tag, "resource")
                resource_tag.set("mime_type", "application/pdf")
                resource_tag.text = resource_url.generate_resource_url(
                    poa_article, poa_article, crossref_config, "text_mining_pdf_pattern"
                )
            if do_set_collection_text_mining_xml(crossref_config) is True:
                item_tag = SubElement(collection_tag, "item")
                resource_tag = SubElement(item_tag, "resource")
                resource_tag.set("mime_type", "application/xml")
                resource_tag.text = resource_url.generate_resource_url(
                    poa_article, poa_article, crossref_config, "text_mining_xml_pattern"
                )


def do_set_collection_text_mining_xml(crossref_config):
    """decide whether to text mining xml resource"""
    if (
        crossref_config.get("text_mining_xml_pattern")
        and crossref_config.get("text_mining_pdf_pattern") != ""
    ):
        return True
    return False


def do_set_collection_text_mining_pdf(poa_article, crossref_config):
    """decide whether to text mining pdf resource"""
    if (
        crossref_config.get("text_mining_pdf_pattern")
        and crossref_config.get("text_mining_pdf_pattern") != ""
        and poa_article.get_self_uri("pdf") is not None
    ):
        return True
    return False


def do_set_collection(poa_article, collection_property, crossref_config):
    """decide whether to set collection tags"""
    # only add text and data mining details if the article has a license
    if not access_indicators.has_license(poa_article):
        return False
    if collection_property == "text-mining":
        if (
            do_set_collection_text_mining_xml(crossref_config) is True
            or do_set_collection_text_mining_pdf(poa_article, crossref_config) is True
        ):
            return True
    return False



import configparser
import json

CONFIG_FILE = "crossref.cfg"


def load_config(config_file=None):
    if not config_file:
        config_file = CONFIG_FILE
    config = configparser.ConfigParser(interpolation=None)
    config.read(config_file)
    return config


def raw_config(config_section, config_file=None):
    """try to load the config section"""
    if not config_file:
        config_file = CONFIG_FILE
    config = load_config(config_file)
    if config.has_section(config_section):
        return config[config_section]
    # default
    return config["DEFAULT"]


def parse_raw_config(raw_config_object):
    """parse the raw config to something good"""
    crossref_config = {}
    boolean_values = []
    int_values = []
    list_values = []

    boolean_values.append("jats_abstract")
    boolean_values.append("face_markup")
    boolean_values.append("crossmark")
    boolean_values.append("elocation_id")
    boolean_values.append("elife_style_component_doi")
    int_values.append("year_of_first_volume")
    list_values.append("contrib_types")
    list_values.append("archive_locations")
    list_values.append("access_indicators_applies_to")
    list_values.append("pub_date_types")
    list_values.append("component_exclude_types")
    list_values.append("crossmark_domains")
    list_values.append("assertion_display_channel_types")

    for value_name in raw_config_object:
        if value_name in boolean_values:
            crossref_config[value_name] = raw_config_object.getboolean(value_name)
        elif value_name in int_values:
            crossref_config[value_name] = raw_config_object.getint(value_name)
        elif value_name in list_values:
            crossref_config[value_name] = json.loads(raw_config_object.get(value_name))
        else:
            # default
            crossref_config[value_name] = raw_config_object.get(value_name)
    return crossref_config
